/* $Id: up.c,v 1.4 2007-03-23 11:42:33 kristaps Exp $ */
/*
Copyright (c) 2007 Kristaps Dzonsons
Latvijas Universitaates Matemaatikas un Informaatikas Instituuts
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are
met:

* Redistributions of source code must retain the above copyright notice,
  this list of conditions and the following disclaimer.
* Redistributions in binary form must reproduce the above copyright
  notice, this list of conditions and the following disclaimer in the
  documentation and/or other materials provided with the distribution.
* Neither the name of the copyright holders nor the names of
  contributors may be used to endorse or promote products derived from
  this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER
OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
#include <sys/types.h>

#include <assert.h>
#include <err.h>
#include <getopt.h>
#include <stdint.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>

#include "nlpcrawl.h"

#define MARK_DIRTY(p, r) do { \
		(p)->state = NLPCDBR_DIRTY; \
		(p)->atime = tms; \
		nlpcdb_dbg("marking stale: %s (%s)", (p)->uri,(r)); \
		if ( ! dryrun) \
			nlpcdb_update(file, hash, (p)); \
	} while (0)

static void
scan(int dryrun, const char *file)
{
	struct nlpcdbr 	rec, match;
	unsigned char	hash[NLPCR_HASHLEN];
	int		rc;
	uint8_t		fields;
	uint64_t	tms;
	time_t		tm;

	/*
	 * TODO: add all modified entities to a transaction so that so
	 * many updates need not take place.
	 */

	assert(NULL != file);
	if ((time_t)-1 == (tm = time(NULL)))
		err(EXIT_FAILURE, "time");
	tms = (uint64_t)tm;

	match.state = NLPCDBR_FRESH | NLPCDBR_SCAN| NLPCDBR_PROC;
	fields = NLPCDBR_FIELD_STATE;

	rc = nlpcdb_nextmatch(file, hash, &rec, &match,
			fields, 0);
	for ( ; rc == 1; rc = nlpcdb_nextmatch(file, hash, &rec, 
				&match, fields, 0)) {
		if (UINT64_MAX == rec.expires) {
			MARK_DIRTY(&rec, "no-cache");
			continue;
		}
		if (0 != rec.expires && rec.expires < tms) {
			MARK_DIRTY(&rec, "http-expire");
			continue;
		}
	}
	assert(0 != rc);
}

static void
usage(void)
{
	extern char 	*__progname;
	fprintf(stderr, "usage: %s [-dvn] [-f file]\n", __progname);
	exit(EXIT_FAILURE);
	/*NOTREACHED*/
}

static void
version(void)
{
	extern char	*__progname;
	fprintf(stdout, "%s version %s\n", __progname, VERSION);
	exit(EXIT_SUCCESS);
	/*NOTREACHED*/
}

int
main(int argc, char *argv[])
{
	extern char 	*optarg;
	extern int 	optind;
	int 		c, dryrun;
	char 		*db;

	db = NLPC_DEFAULT_DB;
	dryrun = 0;

	while ((c = getopt(argc, argv, "dnvf:")) != -1)
		switch (c) {
		case ('n'):
			dryrun = 1;
			break;
		case ('v'):
			version();
			/*NOTREACHED*/
		case ('d'):
			nlpcdb_dbg_lvl++;
			break;
		case ('f'):
			db = optarg;
			break;
		default:
			usage();
			/*NOTREACHED*/
		}
	argc -= optind;
	argv += optind;

	nlpcdb_setup(db);
	scan(dryrun, db);
	nlpcdb_close(db);

	return(EXIT_SUCCESS);
}
